/*
********************************************************************************
*             FT81x simple library for PIC MCU                                 *
********************************************************************************
Copyright (c) Future Technology Devices International / BridgeTek

THIS SOFTWARE IS PROVIDED BY FUTURE TECHNOLOGY DEVICES INTERNATIONAL LIMITED "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
FUTURE TECHNOLOGY DEVICES INTERNATIONAL LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES LOSS OF USE, DATA, OR PROFITS OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

FTDI DRIVERS MAY BE USED ONLY IN CONJUNCTION WITH PRODUCTS BASED ON FTDI PARTS.

FTDI DRIVERS MAY BE DISTRIBUTED IN ANY FORM AS LONG AS LICENSE INFORMATION IS NOT MODIFIED.

IF A CUSTOM VENDOR ID AND/OR PRODUCT ID OR DESCRIPTION STRING ARE USED, IT IS THE
RESPONSIBILITY OF THE PRODUCT MANUFACTURER TO MAINTAIN ANY CHANGES AND SUBSEQUENT WHQL
RE-CERTIFICATION AS A RESULT OF MAKING THESE CHANGES.

********************************************************************************
* This code is provided as an example only and is not guaranteed by FTDI.
* FTDI accept no responsibility for any issues resulting from its use.
* The developer of the final application incorporating any parts of this
* sample project is responsible for ensuring its safe and correct operation
* and for any consequences resulting from its use.
*
* Rev       Date        Author          Comment
********************************************************************************
* 1.0       June 2017	G Brown         Initial Release
* 1.1       Feb  2018   G Brown         Moved oscillator config #Pragmas to here from EVE_Layer
*  *****************************************************************************
*/

#include <xc.h>
#include "Library.h" 
#include "FT8xx.h"
#include <string.h>
#include <stdint.h> // for Uint8/16/32 and Int8/16/32 data types

#define _XTAL_FREQ 12000000      // Required for _delay() function, internal OSC Max

// PIC configuration definitions

// IF INTERNAL OSCILLATOR (NO CRYSTAL))
//#pragma config FOSC = INTRCCLK  // Oscillator Selection bits (INTOSC oscillator: CLKOUT function on RA4/OSC2/CLKOUT pin, I/O function on RA5/OSC1/CLKIN)

#pragma config IESO = ON       // Internal External Switchover enable bit 
#pragma config FCMEN = ON       // Fail-Safe Clock Monitor Enabled bit 
#pragma config PRICLKEN = ON       // 
#pragma config PLLCFG = ON       // Fail-Safe Clock Monitor Enabled bit (Fail-Safe Clock Monitor is enabled)
#pragma config FOSC = 0x03       // Fail-Safe Clock Monitor Enabled bit (Fail-Safe Clock Monitor is enabled)
#pragma config WDTEN = OFF                                                      // Watchdog Timer Enable bit 
#pragma config PWRTEN = OFF                                                     // Power-up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = EXTMCLR                                                  // MCLR Pin Function Select bit (MCLR pin function is MCLR)
#pragma config CP0 = OFF                                                        // Code Protection Block 0 
#pragma config CP1 = OFF                                                        // Code Protection Block 1 
#pragma config CP2 = OFF                                                        // Code Protection Block 2 
#pragma config CP3 = OFF                                                        // Code Protection Block 3 
#pragma config CPB = OFF                                                        // Boot Block Code Protection bit (Boot block (000000-0007FFh) not code-protected)
#pragma config CPD = OFF                                                        // Data EEPROM Code Protection bit (Data EEPROM not code-protected)
#pragma config BOREN = ON                                                       // Brown-out Reset Selection bits (BOR enabled)

//############################################################################## 
//##############################################################################
//                                MCU LAYER 
//##############################################################################  
//##############################################################################
   
// This is the MCU specific section and contains the functions which talk to the
// PIC registers. If porting the code to a different PIC or to another MCU, these
// should be modified to suit the registers of the selected MCU.

// ------------------- MCU specific initialisation  ----------------------------
void MCU_Init(void)
{
    ANSELA = 0x00;
    ANSELB = 0x00;
    ANSELC = 0x00;
    ANSELD = 0x00;
        
    // Port pin set-up
    TRISCbits.TRISC7 = 0;                                                       // CS
    TRISCbits.TRISC6 = 0;                                                       // PD pin
    TRISCbits.TRISC3 = 0;                                                       // SCK 
    TRISCbits.TRISC4 = 1;                                                       // SDI (MISO)
    TRISCbits.TRISC5 = 0;                                                       // SDO (MOSI) 
    
    TRISDbits.TRISD7 = 1;                                                       // RXD INPUT
    TRISDbits.TRISD6 = 0;                                                       // TXD OUTPUT
    TRISDbits.TRISD5 = 1;                                                       // RTS INPUT (FROM FT232 RTS))
    TRISDbits.TRISD4 = 0;                                                       // CTS OUTPUT (TO FT232 CTS)    
       
    LATBbits.LATB0 = 1;
    
    // SPI 1 set-up
    SSP1CON1bits.SSPEN  = 0;                                                    //Disable SPI1
    
    SSP1CON1bits.WCOL   = 0;
    SSP1CON1bits.SSPOV  = 0;
    SSP1CON1bits.CKP    = 0;        
    SSP1CON1bits.SSPM3 = 0;
    SSP1CON1bits.SSPM2 = 0;
    SSP1CON1bits.SSPM1 = 0;
    SSP1CON1bits.SSPM0 = 1;                                                     

    SSP1STATbits.SMP = 0;
    SSP1STATbits.CKE = 1;
    SSP1STATbits.BF  = 0;

    SSP1CON1bits.SSPEN  = 1;                                                    // Enable SPI1 after configuration
}

// ########################### GPIO CONTROL ####################################

// --------------------- Chip Select line low ----------------------------------
void MCU_CSlow(void)
{
    LATCbits.LATC7 = 0;                                                         // CS# line low
    Nop();
}  

// --------------------- Chip Select line high ---------------------------------
void MCU_CShigh(void)
{
    Nop();
    LATCbits.LATC7 = 1;                                                         // CS# line high
}

// -------------------------- PD line low --------------------------------------
void MCU_PDlow(void)
{
    LATCbits.LATC6 = 0;                                                         // PD# line low
}

// ------------------------- PD line high --------------------------------------
void MCU_PDhigh(void)
{
    LATCbits.LATC6 = 1;                                                         // PD# line high
}

// ################################# SPI #######################################

// --------------------- SPI Send and Receive ----------------------------------
uint8_t MCU_SPIReadWrite(uint8_t DataToWrite)
{
    uint8_t DataRead = 0;
    
    SSP1BUF = (DataToWrite);                                                    // Write data to SPI data register                          
    while(!SSP1STATbits.BF);                                                    // Wait for completion of the SPI transfer
    DataRead = SSP1BUF;                                                         // Get the value clocked in from the FT8xx
    
    return DataRead;
}

void MCU_UART_Init(void)
{
    TXSTA2bits.SYNC = 0;                                                        /* asynchronous */
    RCSTA2bits.SPEN = 1;                                                        /* enable serial port pins */
    RCSTA2bits.CREN = 1;                                                        /* enable reception */
    RCSTA2bits.SREN = 0;                                                        /* no effect */
    PIE3bits.TX2IE = 0;                                                         /* disable tx interrupts */
    PIE3bits.RC2IE = 0;                                                         /* disable rx interrupts */
    TXSTA2bits.TX9 = 0;                                                         /* 8- or 9-bit transmission */
    RCSTA2bits.RX9 = 0;                                                         /* 8- or 9-bit reception */
    TXSTA2bits.TXEN = 1;                                                        /* enable the transmitter */
    TXSTA2bits.BRGH2 = 0;                                                       //1;  PLL
    BAUDCON2bits.BRG16 = 0;   
    
    // Baud rate = 48,000,000/(64(n+1))
    // 48000000 / (64(12+1)) = 57692 baud  
    SPBRG2 = 0x0C;
    
    // Baud rate = 48,000,000/(64(n+1))
    // 48000000 / (64(2+1)) = 250000 baud 
    //SPBRG2 = 0x02;
}

void MCU_UART_Tx(uint8_t SerialTxByte)
{
    uint8_t PortValue = 0;
    
   // while( (!PIR3bits.TX2IF) || (PORTDbits.RD5)  )   // TX2IF IS 1 WHEN EMPTY
    //{
   // }
    while (!PIR3bits.TX2IF)
    {
    }
    TXREG2 = SerialTxByte;
}

uint8_t MCU_UART_Rx(void)
{
    uint8_t SerialRxByte = 0x00;
    
    while(!PIR3bits.RC2IF)
    {
    }
    SerialRxByte = RCREG2;
    
    return SerialRxByte;
    
}

void MCU_Delay_20ms(void)
{
    __delay_ms(20);                                                             // gives 20ms due to x4 PLL
    __delay_ms(20);                                                             // gives 20ms due to x4 PLL
    __delay_ms(20);                                                             // gives 20ms due to x4 PLL
    __delay_ms(20);                                                             // gives 20ms due to x4 PLL
}

void MCU_Delay_500ms(void)
{
    uint8_t dly = 0;                                                      
    
    for(dly =0; dly < 100; dly++)
    {
        __delay_ms(20);
    }
}

